package everisStocks

import org.apache.spark.sql.SparkSession
import org.apache.log4j.{Level, LogManager}

trait StocksEndMarketTrait {
  def symbols: List[String]
}

object StocksEndMarket extends App {

  val log = LogManager.getRootLogger
  log.setLevel(Level.INFO)

  log.info(s"----> TFG - Running albguti's StocksEndMarket app with SPARK-Scala. <----\n")

  System.setProperty("HADOOP_USER_NAME","albguti")

  val spark = SparkSession
    .builder()
    .appName("Stocks end of market day")
    .master("yarn-cluster")
    .enableHiveSupport()
    .config("spark.sql.orc.impl", "native")
    .config("spark.sql.parquet.binaryAsString","true")
    .config("spark.sql.parquet.writeLegacyFormat","true")
    .getOrCreate()

  final val symbols: List[String] = List("aapl", "amd", "amzn", "csco", "fb", "googl", "ibm", "intc", "msft", "qcom")

  symbols.foreach { symbol =>
    log.info(s"----> Closing market for $symbol.")

    val symbol_orc: String = symbol+"_orc"
    val symbol_parq: String = symbol+"_parq"
    val symbol_batch: String = symbol+"_batch"

    val sqlDF = spark.sql(s"SELECT A.date, A.open, A.high, A.low, A.close, A.adj_close, A.volume FROM (SELECT cast(date_format(cast((closetime/1000) as timestamp), 'y-MM-d') as string) AS date, open AS open, high AS high, low AS low, close AS close, close AS adj_close, latestvolume AS volume, latestsource, closetime FROM landing_nifi_realtime.land_$symbol_orc WHERE latestsource='Close' ORDER BY closetime DESC LIMIT 1) A")
    sqlDF.write.mode("append").format("hive").saveAsTable(s"staging_$symbol.$symbol_batch")
    log.info(s"----> Following dataframe appended to staging_$symbol.$symbol_batch Hive's table.")

    spark.sql(s"TRUNCATE TABLE landing_nifi_realtime.land_$symbol_orc")
    log.info(s"----> landing_nifi_realtime.land_$symbol_orc truncated.")

    spark.sql(s"TRUNCATE TABLE landing_nifi_realtime.land_$symbol_parq")
    log.info(s"----> landing_nifi_realtime.land_$symbol_parq truncated.")
    log.info(s"----> Market closed for $symbol.")
  }
  log.info(s"----> StocksEndMarket finalized.")
}

/*
spark2-submit --class everisStocks.StocksEndMarket --master yarn --deploy-mode cluster /home/cloudera/IdeaProjects/ReadStocksStreaming/target/scala-2.11/readstocksstreaming_2.11-0.1.jar
spark2-submit --class everisTests.StocksEndMarketTests --master yarn --deploy-mode cluster /home/cloudera/IdeaProjects/ReadStocksStreaming/target/scala-2.11/readstocksstreaming_2.11-0.1.jar
*/